// PluginTestDlg.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "CustomPlugin.h"
#include "PluginTestDlg.h"
#include "dbg.h"
#include "Mach3ObjectModel.h"

#include "KeyHook.h"

#include "Mach3DRO.h"
#include "Mach3Button.h"
#include "Mach3Led.h"

#include "CMyScriptObject.h"
#include ".\plugintestdlg.h"
extern CMyScriptObject scripter;

// ID of the notification message that will be sent to us
#define WM_MY_MESSAGE (WM_APP + 100)


// CPluginTestDlg dialog

IMPLEMENT_DYNAMIC(CPluginTestDlg, CDialog)
CPluginTestDlg::CPluginTestDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CPluginTestDlg::IDD, pParent)
{
	//  Default Constructor
}

CPluginTestDlg::~CPluginTestDlg()
{
	// Defualt Destructor
}

void CPluginTestDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}


BEGIN_MESSAGE_MAP(CPluginTestDlg, CDialog)
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_BN_CLICKED(IDCANCEL, OnBnClickedCancel)
	ON_BN_CLICKED(IDC_RESET, OnBnClickedReset)
	ON_BN_CLICKED(IDC_LOAD_RR, OnBnClickedLoadRr)
	ON_BN_CLICKED(IDC_RUN_RR, OnBnClickedRunRr)
	ON_BN_CLICKED(IDC_HOOK_INSTALL, OnBnClickedHookInstall)
	ON_BN_CLICKED(IDC_HOOK_REMOVE, OnBnClickedHookRemove)
	ON_MESSAGE(WM_MY_MESSAGE, OnMyMessage) // Maps our message
END_MESSAGE_MAP()


// Handler of WM_MY_MESSAGE notification
LRESULT CPluginTestDlg::OnMyMessage(WPARAM wParam, LPARAM lParam)
{
    // The struct to receive information
    KEYRESULT kr; 

    // Information that we need to extract are event type and printable character
    UINT nMask = KH_MASK_EVENTTYPE | KH_MASK_PRINTABLECHAR;
 
    // This function extracts event details
    LRESULT lRes = GetKeyEventResult(wParam, lParam, &kr, nMask);
    ASSERT(lRes == KH_OK);  // Check if passed

    // We only display key-strokes that produce printable characters
    if (kr.chPrintableChar != 0)
    {
        CString sEvent;
        if (kr.iKeyEvent == KH_KEY_DOWN)
            sEvent = _T("Key Down");
        else if (kr.iKeyEvent == KH_KEY_UP)
            sEvent = _T("Key Up");
        else if (kr.iKeyEvent == KH_KEY_REPEAT)
            sEvent = _T("Key Repeat");
        else
            ASSERT(FALSE); // will never happen

        CString sMsg;
        sMsg.Format(_T("Event: %s\nCharacter:%c"), sEvent, kr.chPrintableChar);
        MessageBox(sMsg); // display the result
    }

    return (LRESULT)0;
}

void CPluginTestDlg::OnBnClickedOk()
{
	DbgMsg(("OnBnClickedOk entry"));
	ShowWindow(SW_HIDE);
	DbgMsg(("OnBnClickedOk exit"));
}

void CPluginTestDlg::OnBnClickedCancel()
{
	DbgMsg(("OnBnClickedCancel entry"));
	ShowWindow(SW_HIDE);
	DbgMsg(("OnBnClickedCancel exit"));
}

void CPluginTestDlg::OnBnClickedReset()
{
	DoButton(MACH3_RESET);    // Reset (1021)
}

void CPluginTestDlg::OnBnClickedLoadRr()
{
	Mach3ObjectModelStartup();
	//scripter.LoadFile("C:\\Mach3\\Gcode\\RoadRunner.tap");
	LoadGcodeFile("C:\\Mach3\\Gcode\\RoadRunner.tap");
	Mach3ObjectModelShutdown();
}

void CPluginTestDlg::OnBnClickedRunRr()
{
	DoButton(CYCLE_START);    // Cycle Start (1000)
}

void CPluginTestDlg::OnBnClickedHookInstall()
{
	DbgMsg(("keyboardHook Installed"));
	
	// Install the hook
    LRESULT lRes = InstallKeyHook();
    ASSERT(lRes == KH_OK);  // Check if passed

    // Prepare the KEYENTRY struct
    KEYENTRY ke;
    ke.nMessage = WM_MY_MESSAGE; // Our message ID
    ke.hCallWnd = m_hWnd; // Send message to this window
    ke.hHookWnd = 0; // Capture key-strokes occurred in any windows
    ke.iCombKeys = 0; // Combination key states do not matter
    ke.iIndicators = 0; // Caps-lock, Num-lock, Scroll-lock on/off states do not matter
    ke.iKeyEvent = KH_KEY_DOWN | KH_KEY_UP; // Capture key-down and key-up events
    ke.iMinVKCode = 0; // Capture all keys regardless of their virtual key codes
    ke.iMaxVKCode = 255;

    // Add the entry to the hook
    lRes = AddKeyEntry(&ke);
    ASSERT(lRes == KH_OK);  // Check if passed
}

void CPluginTestDlg::OnBnClickedHookRemove()
{
	DbgMsg(("keyboardHook Removed"));

	// Uninstall the hook to cleanup
    UninstallKeyHook();	
}
